# -*- coding: utf-8 -*-
"""
/***************************************************************************
 ZoomToPostcode
                                 A QGIS plugin
 Zoom the map extent to any UK postcode
                              -------------------
        begin                : 2013-06-16
        copyright            : (C) 2013 by Mining Searches UK
        email                : matthew@miningsearchesuk.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
# Import the PyQt and QGIS libraries
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *
from qgis.gui import *

# Initialize Qt resources from file resources.py
import resources_rc
# Import the code for the dialog
from zoomtopostcodedialog import ZoomToPostcodeDialog
import os.path
import pickle
import urllib2
import zipfile
import tempfile
import sys


class ZoomToPostcode:

    def __init__(self, iface):
        # Save reference to the QGIS interface
        self.iface = iface
        self.canvas = iface.mapCanvas()
        # initialize plugin directory
        self.plugin_dir = os.path.dirname(__file__)
        # initialize locale
        locale = QSettings().value("locale/userLocale")[0:2]
        localePath = os.path.join(self.plugin_dir, 'i18n', 'zoomtopostcode_{}.qm'.format(locale))

        if os.path.exists(localePath):
            self.translator = QTranslator()
            self.translator.load(localePath)

            if qVersion() > '4.3.3':
                QCoreApplication.installTranslator(self.translator)

        # Create the dialog (after translation) and keep reference
        self.dlg = ZoomToPostcodeDialog()

    def initGui(self):
        # Create action that will start plugin configuration
        self.action = QAction(
            QIcon(":/plugins/zoomtopostcode/zoomicon.png"),
            u"Zoom to Postcode", self.iface.mainWindow())
        # connect the action to the run method
        self.action.triggered.connect(self.run)

        # Add toolbar button and menu item
        self.iface.addToolBarIcon(self.action)
        self.iface.addPluginToMenu(u"&Zoom to Postcode", self.action)

    def unload(self):
        # Remove the plugin menu item and icon
        self.iface.removePluginMenu(u"&Zoom to Postcode", self.action)
        self.iface.removeToolBarIcon(self.action)

    # run method that performs all the real work
    def run(self):
        # show the dialog
        self.dlg.show()
        # Run the dialog event loop
        result = self.dlg.exec_()
        # See if OK was pressed
        if result == 1:
            self.check_pkl()


    #Check if a transformation needs to take place
    def check_crs(self):
        canvas = self.canvas
        mapRenderer = canvas.mapRenderer()
        srs = mapRenderer.destinationCrs()
        current_crs = srs.authid()
        return current_crs


    #Transforms point from british nation grid to map crs
    def transform(self, cor):
        canvas = self.canvas
        mapRenderer = canvas.mapRenderer()
        srs = mapRenderer.destinationCrs()
        crsSrc = QgsCoordinateReferenceSystem(27700)
        crsDest = QgsCoordinateReferenceSystem(srs)
        xform = QgsCoordinateTransform(crsSrc, crsDest)
        x = int(cor[0])
        y = int(cor[1])
        t_point = xform.transform(QgsPoint(x,y))
        return t_point

    #Check the Pickle postcode dir exists
    def check_pkl(self):
        checkpkl = pklfile = os.path.isdir(os.path.join(os.path.dirname(__file__), 'UK_Postcodes'))
        if checkpkl == True:
            self.postcode_dict()
        else:
            msg = "Postcode files must be downloaded to use this plugin, do you wish to continue?"
            goahead = QMessageBox.question(self.iface.mainWindow(), "Download Message", msg, QMessageBox.Yes, QMessageBox.No)
            if goahead == QMessageBox.Yes:
                self.download_pkl()
            else:
                pass

    #Download the Pickle postcode file to the plugin dir
    def download_pkl(self):
        pcode_path = os.path.join(os.path.dirname(__file__), 'UK_Postcodes')
        if not os.path.exists(pcode_path):
            os.makedirs(pcode_path)
        url = "http://www.miningsearchesuk.com/assets/file/UK_Postcodes.zip"
        baseFile = os.path.basename(url)
        os.umask(0002)
        try:
            req = urllib2.urlopen(url)
            total_size = int(req.info().getheader('Content-Length').strip())
            downloaded = 0
            CHUNK = 256 * 10240
            dlbar = QProgressBar()
            dlbar.setMinimum(0)
            dlbar.setMaximum(total_size)

            zip_temp = tempfile.NamedTemporaryFile(mode='w+b', suffix='.zip', delete=False)
            zip_temp_n = zip_temp.name
            zip_temp.seek(0)
            with open(zip_temp_n, 'wb') as fp:
                while True:
                    dlbar.show()
                    chunk = req.read(CHUNK)
                    downloaded += len(chunk)
                    dlbar.setValue(downloaded)
                    if not chunk:
                        break
                    fp.write(chunk)
        except urllib2.HTTPError, e:
            QMessageBox.information(self.iface.mainWindow(), "HTTP Error", "Unable to download file")
        pcode_zip = zipfile.ZipFile(zip_temp)
        pcode_zip.extractall(pcode_path)
        zip_temp.close()
        self.run()

    #Create dictonary of postcodes from correct Pickle file
    def postcode_dict(self):
        try:
            input_pcode = self.dlg.ui.lineEdit.text().replace(' ', '')
            if input_pcode[1].isdigit():
                find_pkl = str(input_pcode[:1] + ".pkl")
            else:
                find_pkl = str("UK_Postcodes\\" + input_pcode[:2] + ".pkl")
            pklfile = open(os.path.join(os.path.dirname(__file__), find_pkl), 'rb')
            pcode_dict = pickle.load(pklfile)
            pklfile.close()
            self.zoomto(pcode_dict)
        except (KeyError, IndexError, IOError):
            QMessageBox.information(self.iface.mainWindow(), "Invalid postcode", "The postcode you entered was not found")


    #Find the coordinates for postcode in the dictonary
    def zoomto(self, pcode_dict):
        current_crs = self.check_crs()
        input_pc = self.dlg.ui.lineEdit.text()
        input_pc_fmt = str(input_pc).replace(' ', '').upper()
        find = pcode_dict[input_pc_fmt]
        x = find[0]
        y = find[1]
        if current_crs != "EPSG:27700":
            cor = (x,y)
            point = self.transform(cor)
            self.update_canvas(point)
        else:
            point = (x,y)
            self.update_canvas(point)

    #Update the canvas and add vertex marker
    def update_canvas(self, point):
        x = point[0]
        y = point[1]
        canvas = self.canvas
        scale = 120
        rect = QgsRectangle(float(x)-scale,float(y)-scale,float(x)+scale,float(y)+scale)
        canvas.setExtent(rect)
        self.marker = QgsVertexMarker(canvas)
        self.marker.setIconSize(11)
        self.marker.setPenWidth(2)
        self.marker.setCenter(QgsPoint(int(x),int(y)))
        canvas.refresh()
        canvas.mapCanvasRefreshed.connect(self.remove_marker)

    #Remove vertex marker
    def remove_marker(self):
        canvas = self.canvas
        self.marker.hide()
        canvas.scene().removeItem(self.marker)









